<?php

namespace App\Http\Controllers;

use App\AdminWarehouse;
use App\Company;
use App\ItemName;
use App\WareHouse;
use App\WarehouseTransfer;
use Carbon\Carbon;
use Illuminate\Http\Request;

class WarehouseTransferController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $warehouseTransfers = WarehouseTransfer::whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('warehouse_transfers')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('invoice_no', 'desc')->get();


        return view('crusher.trans_warehouse_transfer.index', compact('warehouseTransfers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();

        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestInvoiceNo = WarehouseTransfer::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        // Calculate the new quotation number
        if ($latestInvoiceNo) {
            $lastInvoiceNo = (int)substr($latestInvoiceNo->invoice_no, -4); // Extract the last 4 digits
            $newNumber = $lastInvoiceNo + 1;
            $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
            $newInvoiceNo = '0001';
        }

        return view('crusher.trans_warehouse_transfer.create', compact('newInvoiceNo', 'items', 'warehouses'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $latestNo = WarehouseTransfer::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        if ($latestNo) {
        $latestInvoiceNo = $latestNo->invoice_no;

        $latestNumber = (int)preg_replace('/[^0-9]/', '', $latestInvoiceNo);

        $newNumber = $latestNumber + 1;

        $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
        $newInvoiceNo = '0001';
        }
        try {
            $validatedData = request()->validate([
                // 'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'item_name_id' => 'nullable|array',
                'item_name_id.*' => 'nullable|exists:item_names,id',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'warehouse_from' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'warehouse_to' => 'nullable|array',
                'warehouse_to.*' => 'nullable|exists:admin_warehouses,id',
            ]);
            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['item_name_id']); $i++)
            {
                $data['invoice_no'] = $newInvoiceNo;
                $data['date'] = $validatedData['date'];
                $data['item_name_id'] = $validatedData['item_name_id'][$i];
                $data['item_description'] = $validatedData['item_description'][$i];
                $data['warehouse_from'] = $validatedData['warehouse_from'][$i];
                $data['warehouse_to'] = $validatedData['warehouse_to'][$i];
                $data['branch_id'] = session()->get('selected_branch');
                WarehouseTransfer::create($data);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('software.warehouse_transfers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($warehouseTransfer)
    {
        $warehouseTransfer = WarehouseTransfer::withTrashed()->find($warehouseTransfer);
        $warehouseTransfer = $warehouseTransfer->withTrashed()->with('warehouseFrom', 'warehouseTo', 'item')->where('invoice_no', $warehouseTransfer->invoice_no)->where('branch_id', session()->get('selected_branch'))->get();
        $company = Company::where('user_id', auth()->id())->first();
        return view('crusher.trans_warehouse_transfer.view', compact('warehouseTransfer', 'company'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\WarehouseTransfer  $warehouseTransfer
     * @return \Illuminate\Http\Response
     */
    public function edit(WarehouseTransfer $warehouseTransfer)
    {
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        // $warehouseTransfers = $warehouseTransfer->with('warehouseFrom', 'warehouseTo', 'item')->where('invoice_no', $warehouseTransfer->invoice_no)->get();

        $warehouseTransfers = WarehouseTransfer::where('branch_id', session()->get('selected_branch'))
        ->with(['warehouseFrom', 'warehouseTo', 'item'])
        ->where('invoice_no', $warehouseTransfer->invoice_no)
        ->get();


        return view('crusher.trans_warehouse_transfer.edit', compact('items', 'warehouses', 'warehouseTransfers'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\WarehouseTransfer  $warehouseTransfer
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, WarehouseTransfer $warehouseTransfer)
    {
        try {
            $validatedData = request()->validate([
                'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'item_name_id' => 'nullable|array',
                'item_name_id.*' => 'nullable|exists:item_names,id',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'warehouse_from' => 'nullable|array',
                'warehouse_from.*' => 'nullable|exists:admin_warehouses,id',
                'warehouse_to' => 'nullable|array',
                'warehouse_to.*' => 'nullable|exists:admin_warehouses,id',
            ]);
            // Iterate over the item descriptions
            $warehouseTransfer->with('warehouseFrom', 'warehouseTo', 'item')->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $warehouseTransfer->invoice_no)->forceDelete();
            for ($i = 0; $i < count($validatedData['item_name_id']); $i++)
            {
                $data['invoice_no'] = $validatedData['invoice_no'];
                $data['date'] = $validatedData['date'];
                $data['item_name_id'] = $validatedData['item_name_id'][$i];
                $data['item_description'] = $validatedData['item_description'][$i];
                $data['warehouse_from'] = $validatedData['warehouse_from'][$i];
                $data['warehouse_to'] = $validatedData['warehouse_to'][$i];
                $data['branch_id'] = session()->get('selected_branch');
                WarehouseTransfer::create($data);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.warehouse_transfers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\WarehouseTransfer  $warehouseTransfer
     * @return \Illuminate\Http\Response
     */
    public function destroy(WarehouseTransfer $warehouseTransfer)
    {
        //
    }

    public function softDelete()
    {
        $transfer = WarehouseTransfer::where('branch_id', session()->get('selected_branch'))->where('invoice_no', request()->invoice_no)->get();

        if ($transfer->isEmpty()) {
            flash()->addError('No records found for the given invoice number.');
            return redirect()->route('crusher.warehouse_transfers.index');
        }

        foreach ($transfer as $creditNote) {
            $creditNote->delete();
        }

        flash()->addSuccess('Success');
        return redirect()->route('software.warehouse_transfers.index');
    }
}
