<?php

namespace App\Http\Controllers;

use App\AdminJournalVoucher;
use App\AdminOtherAccount;
use App\Company;
use App\Customer;
use App\Expense;
use App\Quotation;
use App\QuotationExpense;
use App\StaffAndSalesman;
use App\Supplier;
use App\OtherAccountBuyer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Permission;
use App\Branches;

class AdminJournalVoucherController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $journalVouchers = AdminJournalVoucher::with('account')->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('admin_journal_vouchers')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('invoice_no', 'desc')->get();



        return view('crusher.trans_journal_voucher.index', compact('journalVouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }

        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestVoucher = AdminJournalVoucher::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestVoucher) {
            $lastJVNo = (int)substr($latestVoucher->invoice_no, -4); // Extract the last 4 digits
            $newNumber = $lastJVNo + 1;
            $newJVNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
            $newJVNo = '0001';
        }

        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        //         $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        return view('crusher.trans_journal_voucher.create', compact('customers', 'newJVNo', 'accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request);
        $latestNo = AdminJournalVoucher::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        if ($latestNo) {
        $latestInvoiceNo = $latestNo->invoice_no;

        $latestNumber = (int)preg_replace('/[^0-9]/', '', $latestInvoiceNo);

        $newNumber = $latestNumber + 1;

        $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
        $newInvoiceNo = '0001';
        }
        try {
            $validatedData = $request->validate([
                // 'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'account_name' => 'nullable|array',
                'account_name.*' => 'nullable',
                'narration' => 'nullable|array',
                'narration.*' => 'nullable|string',
                'type' => 'nullable|array',
                'type.*' => 'nullable|in:credit,debit',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable',
                // 'name' => 'nullable|string',
                // 'phone' => 'nullable|string',
                // 'email' => 'nullable|string',
                // 'vat_number' => 'nullable|string',
                // 'address' => 'nullable|string',
                // 'type' => 'nullable|string',
            ]);
            $branch_id = session()->get('selected_branch');

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['account_id']); $i++)
            {
                $data = [
                    'invoice_no' => $newInvoiceNo,
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'][$i]??'',
                    'account_type' => $validatedData['account_type'][$i]??'',
                    'account_name' => $validatedData['account_name'][$i]??'',
                    'narration' => $validatedData['narration'][$i]??'',
                    'type' => $validatedData['type'][$i]??"",
                    'amount' => $validatedData['amount'][$i]??'',
                    'branch_id' => $branch_id,
                ];

                AdminJournalVoucher::create($data);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.journal_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($journalVoucher)
    {
        $journalVoucher = AdminJournalVoucher::withTrashed()->find($journalVoucher);
        $journalVoucher = $journalVoucher->withTrashed()->with('account')->where('invoice_no', $journalVoucher->invoice_no)->where('branch_id', session()->get('selected_branch'))->get();
        $company = Company::where('user_id', auth()->id())->first();
        return view('crusher.trans_journal_voucher.view', compact('journalVoucher', 'company'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function edit(AdminJournalVoucher $journalVoucher)
    {
         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }

        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestVoucher = AdminJournalVoucher::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestVoucher) {
            $lastJVNo = (int)substr($latestVoucher->invoice_no, -4); // Extract the last 4 digits
            $newJVNo = str_pad($lastJVNo + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newJVNo = '-0001';
        }

        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        //         $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);
;

        // $journalVouchers = $journalVoucher->with('account')->where('invoice_no', $journalVoucher->invoice_no)->get();

        $journalVouchers = AdminJournalVoucher::where('branch_id', session()->get('selected_branch'))
        ->with('account')
        ->where('invoice_no', $journalVoucher->invoice_no)
        ->get();


        return view('crusher.trans_journal_voucher.edit', compact('accounts', 'journalVouchers'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, AdminJournalVoucher $journalVoucher)
    {
        dd($request);
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'account_name' => 'nullable|array',
                'account_name.*' => 'nullable',
                'narration' => 'nullable|array',
                'narration.*' => 'nullable|string',
                'type' => 'nullable|array',
                'type.*' => 'nullable|in:credit,debit',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable',
            ]);
            $branch_id = session()->get('selected_branch');
            $journalVoucher = $journalVoucher->with('account')->where('invoice_no', $journalVoucher->invoice_no)->forceDelete();
            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['account_id']); $i++)
            {
                $data = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'][$i]??'',
                    'account_type' => $validatedData['account_type'][$i]??'',
                    'account_name' => $validatedData['account_name'][$i]??'',
                    'narration' => $validatedData['narration'][$i]??'',
                    'type' => $validatedData['type'][$i]??"",
                    'amount' => $validatedData['amount'][$i]??'',
                    'branch_id' => $branch_id,
                ];

                AdminJournalVoucher::create($data);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.journal_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($journalVoucher)
    {
        $journalVoucher = AdminJournalVoucher::findOrFail($journalVoucher);
        $journalVoucher->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $journalVoucher->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.journal_vouchers.index');
    }
}
