<?php

namespace App\Http\Controllers\Transaction;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;


use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\WareHouse;
use App\ItemName;
use App\SalesReturnVoucher;
use App\SalesReturnVoucherItem;
use App\StaffAndSalesman;
use App\Supplier;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AccountMasterTrack;
use App\TermsAndConditionsSalesReturnVoucher;
use App\Company;
use App\Currency;
use App\PurchaseSalesTransferStockTrack;
use App\WareHouseTrack;

class SalesReturnVoucherController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_return_vouchers = SalesReturnVoucher::withTrashed()->with('account')->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.transactions.sales_return_voucher.index', compact('sales_return_vouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_invoice_vat = SalesReturnVoucher::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice_vat){
            $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
        }else{
            $invoice_no_vat = 1;
        }

        $latest_invoice = SalesReturnVoucher::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice){
            $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

        }else{
            $invoice_no = '1A';
        }

        $salesmen = AccountMaster::where('account_master_type', "Staff-And-Salesman")->where('job_designation', 'salesman')->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();


        $item_names = ItemName::Where('location_id', $location_id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_sales_return_voucher = TermsAndConditionsSalesReturnVoucher::where('location_id', $location_id)->first();
        return view('user.transactions.sales_return_voucher.create', compact('currency','invoice_no_vat','invoice_no', 'warehouses', 'salesmen','account_masters','item_names', 'terms_and_conditions_sales_return_voucher'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());

        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $sales_return_voucher = new SalesReturnVoucher();
            $sales_return_voucher->location_id = $location_id;
            $sales_return_voucher->invoice_no = $request->invoice_no;
            $sales_return_voucher->mode_of_payment = $request->mode_of_payment;
            $sales_return_voucher->date = $request->date;
            // $sales_return_voucher->warehouse_id = $request->warehouse_id;
            $sales_return_voucher->salesman_id = $request->salesman_id;
            if ($request->vat_check == "") {
                $sales_return_voucher->vat_check = 0;
            }else {
                $sales_return_voucher->vat_check = $request->vat_check;
            }
            $sales_return_voucher->account_id = $request->account_id;
            $sales_return_voucher->total_quantity = $request->total_quantity;
            $sales_return_voucher->gross_total = $request->gross_total;
            $sales_return_voucher->discount = $request->discount_amount;
            $sales_return_voucher->vat_percentage = $request->vat_percentage;
            $sales_return_voucher->vat_amount = $request->vat_amount;
            $sales_return_voucher->net_total = $request->net_total;
            $sales_return_voucher->amount_in_words = $request->amount_in_words;
            $sales_return_voucher->customer_id = 0;
            $sales_return_voucher->narrations = $request->narrations;
            $sales_return_voucher->buyer_name = $request->buyer_name;
            $sales_return_voucher->buyer_mobile = $request->buyer_mobile;
            $sales_return_voucher->buyer_email = $request->buyer_email;
            $sales_return_voucher->buyer_address = $request->buyer_address;
            $sales_return_voucher->buyer_vat_number = $request->buyer_vat_number;
            $sales_return_voucher->is_signed = 0;
            $sales_return_voucher->save();

            foreach ($request->item_name_id as $key => $item_name_id) {
                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {

                    $sales_return_voucher_item = new SalesReturnVoucherItem();
                    $sales_return_voucher_item-> sales_return_voucher_id = $sales_return_voucher-> id;
                    $sales_return_voucher_item-> item_name_id = $request->item_name_id[$key];
                    $sales_return_voucher_item-> item_unit_id = $request->item_unit[$key];
                    $sales_return_voucher_item-> description = $request->description[$key];
                    // $sales_return_voucher_item-> color = $request->color[$key];
                    $sales_return_voucher_item-> imei_serial_no = $request->imei_serial_no[$key];
                    $sales_return_voucher_item-> warehouse_id = $purchase_sales_transfer_stock_track->warehouse_id;
                    $sales_return_voucher_item-> quantity = $request->quantity[$key];
                    $sales_return_voucher_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                    $sales_return_voucher_item-> amount = $request->amount[$key];
                    $sales_return_voucher_item-> remarks = $request->remarks[$key];
                    $sales_return_voucher_item-> save();

                    $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity + $request->quantity[$key];
                    $purchase_sales_transfer_stock_track->save();

                }else{
                    return redirect()->back()->withErrors("IMEI Not Found In This Location");
                }

            }



            $account = new AccountMastersAccountsDetail();
            $account->account_master_id = $request->account_id;
            $account->debit = 0;
            $account->credit = $request->net_total;
            $account->date = $request->date;
            $account->voucher_type = "credit-note";
            $account->voucher_no = $request->invoice_no;
            $account->save();

            DB::commit();
            Session::flash('alert-success','Sales Return Voucher created successfully');
            return redirect('/transactions/sales-return-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\SalesReturnVoucher  $saleReturnVoucher
     * @return \Illuminate\Http\Response
     */
    public function show(SalesReturnVoucher $saleReturnVoucher)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\SalesReturnVoucher  $saleReturnVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_return_voucher = SalesReturnVoucher::with('items.name')->where('id', $id)->first();

        if ($sales_return_voucher->vat_check == 1) {

            $invoice_no_vat = $sales_return_voucher->invoice_no;

            $latest_invoice = SalesReturnVoucher::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice){
                $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

            }else{
                $invoice_no = '1A';
            }

        }elseif ($sales_return_voucher->vat_check == 0) {

            $invoice_no = $sales_return_voucher->invoice_no;

            $latest_invoice_vat = SalesReturnVoucher::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice_vat){
                $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
            }else{
                $invoice_no_vat = 1;
            }
        }

        $company = Company::where('id', 1)->first();
        // return $company;
        $salesmen = AccountMaster::where('account_master_type', "Staff-And-Salesman")->where('job_designation', 'salesman')->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_id = $sales_return_voucher->account_id;
        $account = AccountMaster::Find($account_id);

        if ($account->transactions_password != "") {
            $pass_check = "yes";
        }elseif ($account->transactions_password == "") {
            $pass_check = "no";
        }

        $currency = Currency::Where('location_id', $location_id)->first();

        $item_names = ItemName::Where('location_id', $location_id)->get();
        $terms_and_conditions_sales_return_voucher = TermsAndConditionsSalesReturnVoucher::where('location_id', $location_id)->first();
        return view('user.transactions.sales_return_voucher.edit', compact('invoice_no','invoice_no_vat','currency','pass_check','sales_return_voucher', 'company', 'warehouses', 'salesmen','account_masters','item_names', 'terms_and_conditions_sales_return_voucher', 'account'));

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\SalesReturnVoucher  $saleReturnVoucher
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $sales_return_voucher = SalesReturnVoucher::where('id', $id)->first();
            $previous_net_total = (float)$sales_return_voucher->net_total;
            $current_net_total = (float)$request->net_total;
            $account = AccountMastersAccountsDetail::Where('account_master_id', $sales_return_voucher->account_id)->Where('voucher_type', "credit-note")->Where('voucher_no', $sales_return_voucher->invoice_no)->first();
            $sales_return_voucher->location_id = $location_id;
            $sales_return_voucher->invoice_no = $request->invoice_no;
            $sales_return_voucher->mode_of_payment = $request->mode_of_payment;
            $sales_return_voucher->date = $request->date;
            // $sales_return_voucher->warehouse_id = $request->warehouse_id;
            $sales_return_voucher->salesman_id = $request->salesman_id;
            if ($request->vat_check == "") {
                $sales_return_voucher->vat_check = 0;
            }else {
                $sales_return_voucher->vat_check = $request->vat_check;
            }
            $sales_return_voucher->account_id = $request->account_id;
            $sales_return_voucher->total_quantity = $request->total_quantity;
            $sales_return_voucher->gross_total = $request->gross_total;
            $sales_return_voucher->discount = $request->discount_amount;
            $sales_return_voucher->vat_percentage = $request->vat_percentage;
            $sales_return_voucher->vat_amount = $request->vat_amount;
            $sales_return_voucher->net_total = $request->net_total;
            $sales_return_voucher->amount_in_words = $request->amount_in_words;
            $sales_return_voucher->customer_id = 0;
            $sales_return_voucher->narrations = $request->narrations;
            $sales_return_voucher->buyer_name = $request->buyer_name;
            $sales_return_voucher->buyer_mobile = $request->buyer_mobile;
            $sales_return_voucher->buyer_email = $request->buyer_email;
            $sales_return_voucher->buyer_address = $request->buyer_address;
            $sales_return_voucher->buyer_vat_number = $request->buyer_vat_number;
            $sales_return_voucher->save();


            // SalesReturnVoucherItem::where('sales_return_voucher_id', $id)->forcedelete();
            $sale_return_items = SalesReturnVoucherItem::with('purchase_sale_stock')->where('sales_return_voucher_id', $id)->get();
            foreach ($sale_return_items as $item) {
                $purchase_sale_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
                $quantity = $item->quantity;
                $purchase_sale_track->quantity = $purchase_sale_track->quantity - $quantity;
                $purchase_sale_track->save();

                $item->forcedelete();
            }

            foreach ($request->item_name_id as $key => $item_name_id) {
                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {

                    $sales_return_voucher_item = new SalesReturnVoucherItem();
                    $sales_return_voucher_item-> sales_return_voucher_id = $sales_return_voucher-> id;
                    $sales_return_voucher_item-> item_name_id = $request->item_name_id[$key];
                    $sales_return_voucher_item-> item_unit_id = $request->item_unit[$key];
                    $sales_return_voucher_item-> description = $request->description[$key];
                    // $sales_return_voucher_item-> color = $request->color[$key];
                    $sales_return_voucher_item-> imei_serial_no = $request->imei_serial_no[$key];
                    $sales_return_voucher_item-> warehouse_id = $purchase_sales_transfer_stock_track->warehouse_id;
                    $sales_return_voucher_item-> quantity = $request->quantity[$key];
                    $sales_return_voucher_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                    $sales_return_voucher_item-> amount = $request->amount[$key];
                    $sales_return_voucher_item-> remarks = $request->remarks[$key];
                    $sales_return_voucher_item-> save();

                    $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity + $request->quantity[$key];
                    $purchase_sales_transfer_stock_track->save();

                }else{
                    return redirect()->back()->withErrors("IMEI Not Found In This Location");
                }

            }

            // $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->Where('voucher_type', "credit-note")->Where('voucher_no', $sales_return_voucher->invoice_no)->first();
// dd($account);
            $account->account_master_id = $request->account_id;
            if ($previous_net_total > $current_net_total) {
                $diff = $previous_net_total - $current_net_total;
                $diff_calculate =  (float)$account->credit - (float)$diff;
                $account->credit =  number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }
            elseif ($previous_net_total < $current_net_total) {
                $diff = $current_net_total - $previous_net_total;
                $diff_calculate = (float)$account->credit + (float)$diff;
                $account->credit = number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }

            $account->save();

            DB::commit();
            Session::flash('alert-success','Sales Return Voucher Updated successfully');
            return redirect('/transactions/sales-return-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\SalesReturnVoucher  $saleReturnVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(SalesReturnVoucher $saleReturnVoucher)
    {
        //
    }

    public function view($id)
    {
        $user = Auth::user();
        if (!$user->can('view-print-credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_return_voucher = SalesReturnVoucher::withTrashed()->with('items_cancelled.name')->where('id', $id)->first();
        $buyer = AccountMaster::Where('id', $sales_return_voucher->account_id)->first();
        $company = Company::where('location_id', $location_id)->first();;
        // return $company;
        $warehouses = WareHouse::all();
        $salesmen = StaffAndSalesman::where('job_designation', 'salesman')->get();

        $account_masters = [];
        $account_master_tracks = AccountMasterTrack::Where('user_id', auth()->user()->id)->get();
        foreach ($account_master_tracks as $key => $account_master_track){
            $account_masters[] = AccountMaster::Where('location_id', $location_id)->Where('id', $account_master_track->account_master_id)->first();
        }

        $item_names = ItemName::all();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_sales_return_voucher = TermsAndConditionsSalesReturnVoucher::where('location_id', $location_id)->first();
        return view('user.print.sales_return_voucher.index', compact('buyer','currency', 'sales_return_voucher', 'company', 'warehouses', 'salesmen','account_masters','item_names', 'terms_and_conditions_sales_return_voucher'));
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $sales_return_voucher = SalesReturnVoucher::Find($id);
        $net_total = $sales_return_voucher->net_total;
        $account_id = $sales_return_voucher->account_id;

        $account = AccountMastersAccountsDetail::Where('account_master_id', $account_id)->Where('voucher_type', "credit-note")->Where('voucher_no', $sales_return_voucher->invoice_no)->first();
        // $credit = $account->credit;
        // $account->credit = (float)$credit - (float)$net_total;
        $account->delete();

        $sales_return_voucher->delete();

        $sales_return_voucher_items = SalesReturnVoucherItem::Where('sales_return_voucher_id', $id)->get();
        foreach ($sales_return_voucher_items as $item) {
            $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
            if ($purchase_sales_transfer_stock_track) {
                $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity - $item->quantity;
                $purchase_sales_transfer_stock_track->save();
            }else{
                return redirect()->back()->withErrors("IMEI Not Found");
            }

            $item->delete();
        }
        Session::flash('alert-success','Sales Return Voucehr Cancelled successfully');
        return redirect('/transactions/sales-return-vouchers');
    }

    public function signature($id){
        $user = Auth::user();
        if (!$user->can('credit-note-sales-return-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $sales_return = SalesReturnVoucher::Find($id);
        return view('user.transactions.sales_return_voucher.signature', compact('sales_return'));
    }

    public function signatureUpload(Request $request, $id){
        $sales_return = SalesReturnVoucher::Find($id);
        $folderPath = public_path('/user_files/img/client_signs/');
        $image_parts = explode(";base64,", $request->signed);
        $image_type_aux = explode("image/", $image_parts[0]);
        $image_type = $image_type_aux[1];
        $image_base64 = base64_decode($image_parts[1]);
        $file_name = 'salesreturnvoucher-' . $id . '.'.$image_type;
        $file = $folderPath . $file_name;
        file_put_contents($file, $image_base64);

        $sales_return->is_signed = 1;
        $sales_return->signature_path = $file_name;
        $sales_return->save();

        Session::flash('alert-success','Sales Return Voucher Signature Uploaded successfully');
        return redirect('/transactions/sales-return-vouchers');
    }
}
