<?php

namespace App\Http\Controllers\Report;

use App\AccountMaster;
use App\Currency;
use App\Http\Controllers\Controller;
use App\PurchaseInvoice;
use App\PurchaseReturnVoucher;
use App\Services\Utils;
use Illuminate\Http\Request;
use PDF;
use Auth;

class PurchaseReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('purchase-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('invoice_no', '!=', 0 )->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $debit_gross_total = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $debit_discount = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $debit = $debit_gross_total - $debit_discount;
        $debit_decimal = number_format($debit, $decimal, '.', '');
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }

        $purchase_return = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $credit_gross_total = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $credit_discount = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $credit = $credit_gross_total - $credit_discount;
        $credit_decimal = number_format($credit, $decimal, '.','');
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }

        $unsorted_data  = $purchase_invoices->concat($purchase_return);

        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        // dd($sorted_data);

        $total = $debit - $credit;
        $total_decimal = number_format($total, $decimal, '.', '');


        return view('user.reports.purchase_report.index', compact('decimal','sorted_data','from_date','to_date', 'credit_decimal', 'debit_decimal', 'total_decimal'));

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function display()
    {
        $user = Auth::user();
        if (!$user->can('purchase-report')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.reports.purchase_report.select_date');
    }

    public function pdfView($from_date, $to_date){

        $user = Auth::user();
        if (!$user->can('purchase-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('invoice_no', '!=', 0 )->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $debit_gross_total = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $debit_discount = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $debit = $debit_gross_total - $debit_discount;
        $debit_decimal = number_format($debit, $decimal, '.', '');
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }

        $purchase_return = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        $credit_gross_total = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $credit_discount = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $credit = $credit_gross_total - $credit_discount;
        $credit_decimal = number_format($credit, $decimal, '.','');
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }

        $unsorted_data  = $purchase_invoices->concat($purchase_return);

        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        // dd($sorted_data);

        $total = $debit - $credit;
        $total_decimal = number_format($total, $decimal, '.', '');

        $pdf = PDF::loadView('user.reports.purchase_report.list', compact('decimal','sorted_data','from_date','to_date', 'credit_decimal', 'debit_decimal', 'total_decimal'));
        return $pdf->stream();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
